// Copyright (c) 2011-2012 Cloudera, Inc.  All rights reserved.
/*global define: false, $: false */

define([
	"cloudera/Util",
	"cloudera/cmf/include/ProgressBar"
], function(Util, ProgressBar) {
	"use strict";

return function(options) {
	/**
	 * Need to figure out when to update the status bar of the command details page.
	 *
	 * If we need to schedule the update at least twice, then we are looking at a
	 * command details of a command that has not finished yet. Set a flag to
	 * needToUpdateStatusBar true.
	 *
	 * Once the command is complete, switch to the current mode. This should
	 * refresh the status bar. In the Express mode, this should still take effect
	 * because this effectively simulates the presence of a time control.
	 */
	var needToUpdateStatusBar = false;
	var $commandDetailsContainer = $(options.containerSelector);
	var fetchUrl = options.fetchUrl;

	function scheduleNextUpdate() {
		setTimeout(fetchData, 1000);
	}

	function fetchData() {
		$.post(
			fetchUrl,
			function(response) {
				$commandDetailsContainer.html(Util.filterError(response));
				var $overallProgress = $commandDetailsContainer.find(".overallProgress");
				var $commandCompleted =  $commandDetailsContainer.find(".commandCompleted");
				if ($commandCompleted.length === 0) {
					if ($(options.containerSelector).is(":visible")) {
						scheduleNextUpdate();
						// the command is not yet complete.
						needToUpdateStatusBar = true;
					}
				} else {
					$overallProgress.parent().ProgressBar('success');
					if (needToUpdateStatusBar) {
						// This switches to the current mode.
						jQuery.publish("markerDateChanged", [Util.getServerNow(), true]);
					}
				}
			}
		);
	}
	scheduleNextUpdate();
};

});
